package com.fr.plugin.tengyuejz.sso;

import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.general.PropertiesUtils;
import com.fr.general.http.HttpToolbox;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.stable.StringUtils;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.stream.Stream;

import static com.fr.plugin.tengyuejz.sso.CommonUtils.*;

/**
 * @author fr.open
 * @since 2021/7/12
 */
@FunctionRecorder
public class SsoFilter extends AbstractGlobalRequestFilterProvider {

    private static String[] NOT_FILTER = {
            "/decision/file",
            "/decision/resources",
            "/system",
            "/materials.min.js.map",
            "/remote",
            "/login",
            "/login/config"
    };

    private final String apiClientId;

    private final String apiClientSecret;

    private final String apiAuthorize;

    private final String apiGetToken;

    private final String apiGetUser;

    private final String errorOutput;

    private final String systemName;

    private final String bipName;

    private final String telephone;

    public SsoFilter() {
        Properties props = PropertiesUtils.getProperties("xxxx");
        this.apiClientId = getProperty(props, "api.client-id", false);
        this.apiClientSecret = getProperty(props, "api.client-secret", false);
        this.apiAuthorize = getProperty(props, "api.authorize", false);
        this.apiGetToken = getProperty(props, "api.get-token", false);
        this.apiGetUser = getProperty(props, "api.get-user", false);
        this.errorOutput = getProperty(props, "api.error-output", false);
        this.systemName = getProperty(props, "info.system-name", false);
        this.bipName = getProperty(props, "info.bip-name", false);
        this.telephone = getProperty(props, "info.telephone", false);
    }

    @Override
    public void doFilter(HttpServletRequest request, HttpServletResponse response, FilterChain chain) {
        if (isAccept(request)) {
            next(request, response, chain);
            return;
        }

        try {
            String code = request.getParameter("code");
            if (StringUtils.isBlank(code)) {
                String redirectURL = URLEncoder.encode(request.getRequestURL().toString(), "utf-8");
                String authorizeURL = String.format("%s?client_id=%s&redirect_uri=%s&response_type=code", apiAuthorize, apiClientId, redirectURL);
                FineLoggerFactory.getLogger().info("没有在参数中找到code值，跳转到认证登录地址, 认证地址为\"{}\"", authorizeURL);
                response.sendRedirect(authorizeURL);
                return;
            }
            login(getUsername(getToken(code)), request, response);
            next(request, response, chain);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("单点登录处理失败.", e);
            setError(response, e.getMessage());
        }
    }

    @Override
    public String filterName() {
        return "sso";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/*"};
    }

    private boolean isAccept(HttpServletRequest request) {
        String url = request.getRequestURL().toString();
        return isLogin(request) || Stream.of(NOT_FILTER).anyMatch(url::contains);
    }

    private String getToken(String code) throws IOException {
        Map<String, Object> params = new HashMap<>();
        params.put("client_id", apiClientId);
        params.put("client_secret", apiClientSecret);
        params.put("code", code);
        params.put("grant_type", "authorization_code");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/x-www-form-urlencoded");
        String res = HttpToolbox.post(apiGetToken, params, headers);
        FineLoggerFactory.getLogger().info("获取Token接口返回内容: \"{}\"", res);
        JSONObject body = new JSONObject(res);
        if (body.has("access_token")) {
            String token = body.getString("access_token");
            FineLoggerFactory.getLogger().info("获取到的token值为: \"{}\"", token);
            return token;
        }
        throw new RuntimeException("获取token失败");
    }

    private String getUsername(String token) throws IOException {
        String api = String.format("%s?access_token=%s&client_id=%s", apiGetUser, token, apiClientId);
        FineLoggerFactory.getLogger().info("获取用户信息接口访问地址为: \"{}\"", api);
        String res = HttpToolbox.get(api);
        FineLoggerFactory.getLogger().info("获取用户信息接口返回内容: \"{}\"", res);
        JSONObject body = new JSONObject(res);
        if (body.has("spRoleList")) {
            JSONArray roles = body.getJSONArray("spRoleList");
            if (roles.size() > 0) {
                String username = roles.getString(0);
                FineLoggerFactory.getLogger().info("获取到的用户名为: \"{}\"", username);
                return username;
            } else {
                throw new RuntimeException("系统未授权, 当前用户是\"" + body.getString("displayName") + "\"");
            }
        }
        throw new RuntimeException("访问获取用户信息接口失败");
    }

    private void setError(HttpServletResponse res, String reason) {
        try {
            String URL = String.format(
                    "%s?systemName=%s&bipName=%s&telephone=%s&reason=%s",
                    errorOutput,
                    URLEncoder.encode(systemName, "utf-8"),
                    URLEncoder.encode(bipName, "utf-8"),
                    URLEncoder.encode(telephone, "utf-8"),
                    URLEncoder.encode(reason, "utf-8")
            );
            res.sendRedirect(URL);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
    }

}
